import re
import numpy as np

from prettytable import PrettyTable


def main():
    bin_labels = r"$1.10 < q^2 < 6.00$"

    sample_1             = """+----------+------------------------+------------------------+-----------------------+\n
                              | Variable |         Value          |      Lower Error       |      Upper Error      |\n
                              +----------+------------------------+------------------------+-----------------------+\n
                              |   AFB    | -0.030693939006905927  | -0.0027448398251912667 | 0.0026690127045430573 |\n
                              |    FL    |   0.7641860897390124   | -0.0031519177028962353 |  0.00318403722094425  |\n
                              |    S3    | -0.009793642767841643  | -0.0039500538800350165 |  0.003962284039325228 |\n
                              |    S4    |  -0.11948833798312776  | -0.005018301784506872  |  0.005112053156795224 |\n
                              |    S5    |  -0.14672854148123884  | -0.005077800231656763  | 0.0051383664423323315 |\n
                              |    S7    | -0.0009713979621990259 | -0.005287904531482017  |  0.005275166744474821 |\n
                              |    S8    |  0.003178551357900045  | -0.005257961171467131  |  0.005258879438751017 |\n
                              |    S9    |  0.005757808359963191  | -0.004016272402049568  |  0.004019873761938199 |\n
                              +----------+------------------------+------------------------+-----------------------+""" # GENERATOR LEVEL

    sample_2             = """+----------+-----------------------+------------------------+-----------------------+\n
                              | Variable |         Value         |      Lower Error       |      Upper Error      |\n
                              +----------+-----------------------+------------------------+-----------------------+\n
                              |   AFB    | -0.012550006150333942 | -0.0026956457681952418 | 0.0026965400452885245 |\n
                              |    FL    |   0.775244817820803   | -0.0030583204477219544 |  0.003052276555621123 |\n
                              |    S3    | -0.004731791649125892 | -0.0035876206689923096 | 0.0035671656402230033 |\n
                              |    S4    |  -0.12872329698226329 | -0.0054353577466807685 |  0.005452103886085265 |\n
                              |    S5    |  -0.13102847916212362 | -0.005094396678852025  |  0.005083576918431511 |\n
                              |    S7    |  0.014829855067600298 | -0.005251337297972407  |  0.005258433080994564 |\n
                              |    S8    |  0.011919411650166327 | -0.005666331650738382  |  0.005691209183046813 |\n
                              |    S9    | -0.013513044362093558 | -0.0036909184587611017 | 0.0036915155625497006 |\n
                              +----------+-----------------------+------------------------+-----------------------+""" # MC LEVEL

    variables    = ["AFB", "FL", "S3", "S4", "S5", "S7", "S8", "S9"]
    select_regex = [".*AFB.*\d+", ".*FL.*\d+", ".*S3.*\d+", ".*S4.*\d+", ".*S5.*\d+", ".*S7.*\d+", ".*S8.*\d+", ".*S9.*\d+"]
    value_regex  = "[- ]\d+.\d+"
    sample_table_1   = {'1': {'AFB': None, 'FL': None, 'S3': None, 'S4': None, 'S5': None, 'S7': None, 'S8': None, 'S9': None}}
    sample_table_2   = {'1': {'AFB': None, 'FL': None, 'S3': None, 'S4': None, 'S5': None, 'S7': None, 'S8': None, 'S9': None}}
    samples = [sample_1, sample_2]

    for variable, regex in zip(variables, select_regex):
        sample_1_splitted = samples[0].splitlines()
        for line in sample_1_splitted:
            subresult = re.match(regex, line)
            if subresult != None:
                substring = subresult.group(0)
                result    = re.findall(value_regex, substring)
                value     = np.float64(result[0])
                lower_err = np.float64(result[1])
                upper_err = np.float64(result[2])

                sample_table_1['1'][variable] = (value, lower_err, upper_err)

        sample_2_splitted = samples[1].splitlines()
        for line in sample_2_splitted:
            subresult = re.match(regex, line)
            if subresult != None:
                substring = subresult.group(0)
                result    = re.findall(value_regex, substring)
                value     = np.float64(result[0])
                lower_err = np.float64(result[1])
                upper_err = np.float64(result[2])

                sample_table_2['1'][variable] = (value, lower_err, upper_err)

    info_table = PrettyTable(["Variable", r"|MC reweighted - Gen|", "Lower Error", "Upper Error", "Stat. Significance"])
    for name in variables:
        index = '1'
        delta_value       = np.abs(sample_table_1[index][name][0] - sample_table_2[index][name][0])
        quadratic_lower   = -np.sqrt(sample_table_1[index][name][1]**2 + sample_table_2[index][name][1]**2)
        quadratic_upper   = np.sqrt(sample_table_1[index][name][2]**2 + sample_table_2[index][name][2]**2)
        stat_significance = np.abs(delta_value / quadratic_lower)
        info_table.add_row([name, delta_value, quadratic_lower, quadratic_upper, stat_significance])
    info_table.title = f"{bin_labels}"
    print(info_table)


if __name__ == "__main__":
    main()